#include <stdio.h>
#include <string.h>
#include <conio.h>
#include <ctype.h>

#include "input.h"
#include "game.h"
#include "output.h"

#ifdef SMALL
# define VERBS 27
#else
# define VERBS 30
#endif

static const struct verb_s
{
	const unsigned char len;
	const char *s;
	unsigned char verbnum;
} verbs[VERBS] =
{
	{ 1, "n", V_N },
	{ 1, "s", V_S },
	{ 1, "e", V_E },
	{ 1, "w", V_W },
	{ 1, "i", V_I },
	{ 4, "take", V_TAKE },
	{ 7, "pick up", V_TAKE },
	{ 4, "lift", V_LIFT },
	{ 4, "read", V_READ },
	{ 7, "talk to", V_TALKTO },
	{ 6, "unlock", V_UNLOCK },
	{ 4, "open", V_UNLOCK },
	{ 4, "turn", V_TURN },
	{ 3, "hit", V_HIT },
	{ 5, "punch", V_HIT },
	{ 4, "kick", V_HIT },
	{ 4, "kill", V_KILL },
	{ 4, "pour", V_POUR },
	{ 5, "empty", V_POUR },
	{ 5, "drink", V_DRINK },
	{ 5, "throw", V_THROW },
	{ 7, "look at", V_LOOK },
	{ 4, "look", V_LOOK },
	{ 7, "examine", V_LOOK },
	{ 4, "drop", V_DROP },
	{ 4, "help", V_HELP },
#ifndef SMALL
	{ 7, "license", V_LICENSE },
	{ 7, "licence", V_LICENSE },
#endif
	{ 4, "quit", V_QUIT },
	{ 4, "exit", V_QUIT },
};

#ifdef __C64__
void __fastcall__ getline(void)
{
#define CHRIN 0xFFCF

	__asm__("ldy #$FF");
	__asm__("chrinloop: iny");
	__asm__("jsr %w", CHRIN);
	__asm__("sta %w,y", (unsigned int) input);
	__asm__("cmp #%b", '\n');
	__asm__("bne chrinloop");
	__asm__("lda #0");
	__asm__("sta %w,y", (unsigned int) input);
}
#endif

/* text_input
 *
 * Get some text input from the user.  Pass in the max number of characters
 * to be accepted, and a pointer to a string that is at least as big plus one
 * for the null terminator to hold the string.
 *
 * Returns the number of characters entered if everything went ok
 * and the user pressed return, 0 if the user pressed Escape.
 */

//unsigned char __fastcall__ text_input(unsigned char x, unsigned char y, char
//*text, unsigned char max, unsigned char num_only)
unsigned char __fastcall__ text_input(char *text, unsigned char max, unsigned char num_only)
{
  char c;
  unsigned char i;
  unsigned char x;
  unsigned char y;
 
  //--- Initialize ---//
  i = 0;
  cursor(1);
  //cputsxy(x, y, text);                  // show text
  //gotoxy(x, y); 
  x = wherex();
  y = wherey();
  for(i=0;i<max;i++)
  {
	text[i]='\0';
  }
  i = 0;
  while (1) {
    c = cgetc();
 
    switch(c) {
      case 27:                          // user pressed escape
            text[0] = 27;                   // flag that user pressed escape
            cursor(0);                      // since a blank string and escape 
            return(0);                      // both have a return value of zero
            break;
 
      case 155:                         // user pressed return
            text[strlen(text)] = '\0';
            cursor(0);
            return(i);
            break;
 
      case 126:                         // user pressed backspace 
            if (i != 0) {
              --i;
              if (i == (strlen(text)-1))
                text[i] = '\0';
              else
                text[i] = ' ';
              //cclearxy(x+i, y, 1);
              cputcxy(x+i, y, ' ');
              gotoxy(x+i, y);
            }
            break;
 
      case 30:                          // user pressed left arrow
            if (i != 0) {
              --i;
              gotoxy(x+i, y);
            }
            break;
 
      case 31:                          // user pressed right arrow
            if (i < strlen(text)) {
              ++i;
              gotoxy(x+i, y); 
            }
 
      default:
            if (i == max) continue;                     // maxed out
 
            if (!num_only) {
              if (isprint(c)) {                           // is a printable char
                text[i] = c;
                cputc(c);
                ++i;
                break;
              }
            }
 
            if (isdigit(c)) {                           // is a number
              text[i] = c;
              cputc(c);
              ++i;
              break;
            }
 
            break;
    }
  }
  // end of function
}


unsigned char getinput(void)
{
	while (1)
	{
		NR unsigned char i;
		NR const struct verb_s *verb;

#ifdef __C64__
		PUTS("Your turn?");
		getline();
#else
		fputs("\nYour turn? ", stdout);
		//fgets(input, 38, stdin);
		text_input(input, 28, 0);
		for(i = 0; i < 29; i++)
		{
			input[i]=tolower(input[i]);
		}

		//printf("%s",input);
#endif
		PUTCHAR('\n');

		verb = verbs;
		for (i = VERBS; i; -- i, ++ verb)
		{
			NR unsigned char n;
			NR char *p;

			n = verb->len; /* Verb length */
			p = input + n; /* First character after verb */
			if (!strncmp(input, verb->s, n) &&
			    (!*p || ' ' == *p))
			{
				if (' ' == *p)
				{
					object_string = p + 1;
				}
				else
				{
					object_string = NULL;
				}
				return verb->verbnum;
			}
		}

#ifdef SMALL
		PUTS("What?");
#else
		PUTS("I do not understand.");
#endif
	}
}
